// ============================================================================
// (C) Copyright IBM Corporation 2006, 2007.  All Rights Reserved.
// 
// You may only copy, modify, and distribute these samples internally.
// These samples have not been tested under all conditions and are provided
// to you by IBM without obligation of support of any kind.
//
// IBM PROVIDES THESE SAMPLES "AS IS" SUBJECT TO ANY STATUTORY WARRANTIES THAT
// CANNOT BE EXCLUDED. IBM MAKES NO WARRANTIES OR CONDITIONS, EITHER EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OR
// CONDITIONS OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND
// NON-INFRINGEMENT REGARDING THESE SAMPLES OR TECHNICAL SUPPORT, IF ANY.
// ============================================================================
dojo.provide("zero.data.LazyLoadReadStore");

dojo.require("dojo.data.util.filter");
dojo.require("dojo.data.util.simpleFetch");
dojo.require("dojo.date.stamp");

dojo.declare("zero.data.LazyLoadReadStore", null,{
	constructor: function(/* Object */ keywordParameters){
		//	summary: constructor
		//	keywordParameters: {url: String}
		//	keywordParameters: {typeMap: object)
		//		The structure of the typeMap object is as follows:
		//		{
		//			type0: function || object,
		//			type1: function || object,
		//			...
		//			typeN: function || object
		//		}
		//		Where if it is a function, it is assumed to be an object constructor that takes the 
		//		value of _value as the initialization parameters.  If it is an object, then it is assumed
		//		to be an object of general form:
		//		{
		//			type: function, //constructor.
		//			deserialize:	function(value) //The function that parses the value and constructs the object defined by type appropriately.
		//		}
		
		this._url = keywordParameters.url;  
		this._features = {'dojo.data.api.Read' : true, 'dojo.data.api.Identity' : keywordParameters.identifierAttr};
		this._labelAttr = keywordParameters.labelAttr;
		
		this._storeRefPropName = "_S";  // Default name for the store reference to attach to every item.
	},

	_assertIsItem: function( /* item */ item) {
		if(!this.isItem(item)){ 
			throw new Error("zero.data.LazyLoadReadStore: a function was passed an item argument that was not an item");
		}
	},
	
	getValue: function(	/* item */ item, 
						/* attribute-name-string */ attribute, 
						/* value? */ defaultValue){
		//	summary: 
		//		See dojo.data.api.Read.getValue()
		this._assertIsItem(item);
		this._assertIsAttribute(attribute);
		
		var value = item[attribute]; // Array
		if (dojo.isArray(value)) {
			value = value[0];
		} else if (!value) {
			value = defaultValue;
		}
		return value;
	},

	getValues: function(/* item */ item, 
						/* attribute-name-string */ attribute){
		//	summary: 
		//		See dojo.data.api.Read.getValues()
		
		this._assertIsItem(item);
		this._assertIsAttribute(attribute);
		var value = item[attribute] || []; // Array
		if (!dojo.isArray(value)) {
			value = [value];
		}
		return value;
	},

	_assertIsAttribute: function(/* attribute-name-string */ attribute){
		if(typeof attribute !== "string"){ 
			throw new Error("zero.data.LazyLoadReadStore: a function was passed an attribute argument that was not an attribute name string");
		}
	},

	getAttributes: function(/* item */ item){
		//	summary: 
		//		See dojo.data.api.Read.getAttributes()
		var attributes = [];
		for(var key in item){
			if(key !== this._storeRefPropName) {
				attributes.push(key);
			}
		}
		return attributes; // Array
	},

	hasAttribute: function(	/* item */ item,
							/* attribute-name-string */ attribute) {
		//	summary: 
		//		See dojo.data.api.Read.hasAttribute()
		return this.getValues(item, attribute).length > 0;
	},

	containsValue: function(/* item */ item, 
							/* attribute-name-string */ attribute, 
							/* anything */ value){
		//	summary: 
		//		See dojo.data.api.Read.containsValue()
		var regexp = undefined;
		if(typeof value === "string"){
			regexp = dojo.data.util.filter.patternToRegExp(value, false);
		}
		return this._containsValue(item, attribute, value, regexp); //boolean.
	},

	_containsValue: function(	/* item */ item, 
								/* attribute-name-string */ attribute, 
								/* anything */ value,
								/* RegExp?*/ regexp){
		//	summary: 
		//		Internal function for looking at the values contained by the item.
		//	description: 
		//		Internal function for looking at the values contained by the item.  This 
		//		function allows for denoting if the comparison should be case sensitive for
		//		strings or not (for handling filtering cases where string case should not matter)
		//	
		//	item:
		//		The data item to examine for attribute values.
		//	attribute:
		//		The attribute to inspect.
		//	value:	
		//		The value to match.
		//	regexp:
		//		Optional regular expression generated off value if value was of string type to handle wildcarding.
		//		If present and attribute values are string, then it can be used for comparison instead of 'value'
		var values = this.getValues(item, attribute);
		for(var i = 0; i < values.length; ++i){
			var possibleValue = values[i];
			if(typeof possibleValue === "string" && regexp){
				return (possibleValue.match(regexp) !== null);
			}else{
				//Non-string matching.
				if(value === possibleValue){
					return true; // Boolean
				}
			}
		}
		return false; // Boolean
	},

	isItem: function(/* anything */ something){
		//	summary: 
		//		See dojo.data.api.Read.isItem()
		if(something && something[this._storeRefPropName] === this){
			return true;
		}
		return false; // Boolean
	},

	isItemLoaded: function(/* anything */ something){
		return this.isItem(something); //boolean
	},

	loadItem: function(/* object */ keywordArgs){
		var item = keywordArgs.item;

		var id = this.getIdentity(item);
		if (id) {
			this._fetchByIdentity(keywordArgs, id);
		} else {
			throw new Error("zero.data.LazyLoadReadStore: a function was passed an item that contained no identity");
		}
	},

	getFeatures: function(){
		return this._features; //Object
	},

	getLabel: function(/* item */ item){
		//	summary: 
		//		See dojo.data.api.Read.getLabel()
		if(this._labelAttr && this.isItem(item)){
			return this.getValue(item,this._labelAttr); //String
		}
		return undefined; //undefined
	},

	getLabelAttributes: function(/* item */ item){
		//	summary: 
		//		See dojo.data.api.Read.getLabelAttributes()
		if(this._labelAttr){
			return [this._labelAttr]; //array
		}
		return null; //null
	},

	_fetchItems: function(	/* Object */ keywordArgs, 
							/* Function */ fetchCallback, 
							/* Function */ errorCallback){
		//	summary: 
		//		See dojo.data.util.simpleFetch.fetch()
		var self = this;

		if(this._url){
			var getArgs = {
				url: self._url, 
				handleAs: "json-comment-optional"
			};
			var getHandler = dojo.xhrGet(getArgs);
			getHandler.addCallback(function(data){
				try{
					for(var i in data) {
						data[i][self._storeRefPropName] = self;
					}
					
					fetchCallback(data, keywordArgs);				
				}catch(e){
					errorCallback(e, keywordArgs);
				}
			});
			getHandler.addErrback(function(error){
				errorCallback(error, keywordArgs);
			});
		}else{
			errorCallback(new Error("zero.data.LazyLoadReadStore: No url was provided."), keywordArgs);
		}
	},

	close: function(/*dojo.data.api.Request || keywordArgs || null */ request){
		 //	summary: 
		 //		See dojo.data.api.Read.close()
	},

	getIdentity: function(/* item */ item){
		//	summary: 
		//		See dojo.data.api.Identity.getIdentity()
		var identifier = this._features['dojo.data.api.Identity'];
		return item[identifier];
	},

	fetchItemByIdentity: function(/* Object */ keywordArgs){
		_fetchItemByIdentity(keywordArgs, keywordArgs.identity);
	},

	_fetchByIdentity: function(/* Object */ keywordArgs, /* String */ id) {
		var url = this._url;
		if (id.charAt(0) != '/') {
			url += '/'
		}
		url += id;

		//Need a reference to the store to call back to its structures.
		var self = this;

		// Callback for handling a successful load.
		var onData = function(data){
			data[self._storeRefPropName] = self;
			
			if(keywordArgs.item) {
				// copy the retrieved data into the original item
				dojo.mixin(keywordArgs.item, data);
				data = keywordArgs.item;
			}
			
			//Done processing!  Call the onItem, if any.
			if(keywordArgs.onItem){
				var scope = keywordArgs.scope ? keywordArgs.scope : dojo.global;
				keywordArgs.onItem.call(scope, data);
			}
		};

		//Callback for any errors that occur during load.
		var onError = function(error){
			//Call the onComplete, if any
			if(keywordArgs.onError){
				var scope = keywordArgs.scope ? keywordArgs.scope : dojo.global;
				keywordArgs.onError.call(scope, error);
			}
		};

		//Fire the get and pass the proper callbacks to the deferred.
		var xhrArgs = {
			url: url,
			handleAs: "json-comment-optional"
		};
		var d = dojo.xhrGet(xhrArgs);
		d.addCallbacks(onData, onError);
	},
	
	getIdentityAttributes: function(/* item */ item){
		//	summary: 
		//		See dojo.data.api.Identity.getIdentifierAttributes()
		 
		return this._features['dojo.data.api.Identity'];
	}
});
//Mix in the simple fetch implementation to this class.
dojo.extend(zero.data.LazyLoadReadStore,dojo.data.util.simpleFetch);